@@ -1,3 +1,23 @@
+# $Id: mkinfit.R 59 2010-07-28 12:29:15Z jranke $
+
+# Copyright (C) 2010 Johannes Ranke
+# Contact: mkin-devel@lists.berlios.de
+
+# This file is part of the R package mkin
+
+# mkin is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
+
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+
+# You should have received a copy of the GNU General Public License along with
+# this program. If not, see <http://www.gnu.org/licenses/>
+
 mkinfit <- function(mkinmod, observed,
   parms.ini = rep(0.1, length(mkinmod$parms)),
   state.ini = c(100, rep(0, length(mkinmod$diffs) - 1)), 
@@ -166,10 +186,11 @@ mkinfit <- function(mkinmod, observed,
   }
   fit$errmin <- errmin
 
-  # Calculate dissipation times DT50 and DT90
+  # Calculate dissipation times DT50 and DT90 and formation fractions
   parms.all = c(fit$par, parms.fixed)
   fit$distimes <- data.frame(DT50 = rep(NA, length(obs_vars)), DT90 = rep(NA, length(obs_vars)), 
     row.names = obs_vars)
+  fit$ff <- vector()
   for (obs_var in obs_vars) {
       type = names(mkinmod$map[[obs_var]])[1]  
       if (type == "SFO") {
@@ -177,12 +198,22 @@ mkinfit <- function(mkinmod, observed,
         k_tot = sum(parms.all[k_names])
         DT50 = log(2)/k_tot
         DT90 = log(10)/k_tot
+        for (k_name in k_names)
+        {
+          fit$ff[[sub("k_", "", k_name)]] = parms.all[[k_name]] / k_tot
+        }
       }
       if (type == "FOMC") {
         alpha = parms.all["alpha"]
         beta = parms.all["beta"]
         DT50 = beta * (2^(1/alpha) - 1)
         DT90 = beta * (10^(1/alpha) - 1)
+        ff_names = grep(paste("f_to_", sep="_"), names(parms.all), value=TRUE)
+        for (ff_name in ff_names)
+        {
+          fit$ff[[sub("f_to", obs_var, ff_name)]] = parms.all[[ff_name]]
+        }
+        fit$ff[[paste(obs_var, "sink", sep="_")]] = 1 - sum(parms.all[ff_names])
       }
       if (type == "SFORB") {
         # FOCUS kinetics (2006), p. 60 f
@@ -207,6 +238,10 @@ mkinfit <- function(mkinmod, observed,
         f_90 <- function(t) (SFORB_fraction(t) - 0.1)^2
         DT90.o <- optimize(f_90, c(0.01, 1000))$minimum
         if (abs(DT90.o - max_DT) < 0.01) DT90 = NA else DT90 = DT90.o
+        for (k_out_name in k_out_names)
+        {
+          fit$ff[[sub("k_", "", k_out_name)]] = parms.all[[k_out_name]] / k_1output
+        }
       }
       fit$distimes[obs_var, ] = c(DT50, DT90)
   }
@@ -222,7 +257,7 @@ mkinfit <- function(mkinmod, observed,
   return(fit)
 }
 
-summary.mkinfit <- function(object, data = TRUE, distimes = TRUE, cov = FALSE,...) {
+summary.mkinfit <- function(object, data = TRUE, distimes = TRUE, ff = TRUE, cov = FALSE,...) {
   ans <- FME:::summary.modFit(object, cov = cov)
   ans$diffs <- object$diffs
   if(data) ans$data <- object$data
@@ -230,6 +265,7 @@ summary.mkinfit <- function(object, data = TRUE, distimes = TRUE, cov = FALSE,..
   ans$fixed <- object$fixed
   ans$errmin <- object$errmin 
   if(distimes) ans$distimes <- object$distimes
+  if(ff) ans$ff <- object$ff
   class(ans) <- c("summary.mkinfit", "summary.modFit") 
   return(ans)  
 }
@@ -264,6 +300,12 @@ print.summary.mkinfit <- function(x, digits = max(3, getOption("digits") - 3), .
     print(x$distimes, digits=digits,...)
   }    
 
+  printff <- !is.null(x$ff)
+  if(printdistimes){
+    cat("\nEstimated formation fractions\n")
+    print(data.frame(ff = x$ff), digits=digits,...)
+  }    
+
   printcor <- !is.null(x$cov.unscaled)
   if (printcor){
     Corr <- cov2cor(x$cov.unscaled)
